@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker and Peter Ludemann
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2000-2023, University of Amsterdam
+    Copyright (c)  2000-2024, University of Amsterdam
 			      VU University Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -105,25 +105,62 @@ PlEx_fail(qid_t qid)
 }
 
 
+// TODO: add unit test for unrepresentable string - should throw error
 _SWI_CPP2_CPP_inline
 const std::string
 PlTerm::get_nchars(unsigned int flags) const
-{ if ( is_null() )
-    return "<null-term>";
+{ flags &= ~static_cast<unsigned int>(BUF_STACK|BUF_MALLOC|BUF_ALLOW_STACK);
+  flags |= static_cast<unsigned int>(BUF_DISCARDABLE|CVT_EXCEPTION);
+  char *s = nullptr;
+  size_t len = 0;
   PlStringBuffers _string_buffers;
-  char  *s;
-  size_t len;
-  if ( ! (flags&BUF_MALLOC) )
-    flags |= BUF_STACK;
-  PlEx<int>(get_nchars(&len, &s, flags|CVT_EXCEPTION));
-  if ( flags&BUF_MALLOC )
-    { std::string result(s, len);
-      Plx_free(s);
-      return result;
-    }
-  return std::string(s, len);
+  if ( _get_nchars(&len, &s, flags) )
+    return std::string(s, len);
+  PlEx_fail();
+  return "<---get_nchars error --->"; // Should never get here
 }
 
+_SWI_CPP2_CPP_inline
+const std::wstring
+PlTerm::get_wchars(unsigned int flags) const
+{ flags &= ~static_cast<unsigned int>(BUF_STACK|BUF_MALLOC|BUF_ALLOW_STACK);
+  flags |= static_cast<unsigned int>(BUF_DISCARDABLE|CVT_EXCEPTION);
+  pl_wchar_t *s;
+  size_t len;
+  PlStringBuffers _string_buffers;
+  if ( _get_wchars(&len, &s, flags) )
+    return std::wstring(s, len);
+  PlEx_fail();
+  return L"<---get_wchars error --->"; // Should never get here
+}
+
+#define _MUST_BE_TYPE(must_be_name, is_test, type_name)  \
+_SWI_CPP2_CPP_inline \
+void \
+PlTerm::must_be_name() const \
+{ if ( !is_test() ) \
+    throw PlTypeError(type_name, *this); \
+}
+
+_MUST_BE_TYPE(must_be_attvar,         is_attvar,         "attvar")
+_MUST_BE_TYPE(must_be_variable,       is_variable,       "variable")
+_MUST_BE_TYPE(must_be_ground,         is_ground,         "ground")
+_MUST_BE_TYPE(must_be_atom,           is_atom,           "atom")
+_MUST_BE_TYPE(must_be_integer,        is_integer,        "integer")
+_MUST_BE_TYPE(must_be_string,         is_string,         "string")
+_MUST_BE_TYPE(must_be_atom_or_string, is_atom_or_string, "atom or string")
+_MUST_BE_TYPE(must_be_float,          is_float,          "float")
+_MUST_BE_TYPE(must_be_rational,       is_rational,       "rational")
+_MUST_BE_TYPE(must_be_compound,       is_compound,       "compound")
+_MUST_BE_TYPE(must_be_callable,       is_callable,       "callable")
+_MUST_BE_TYPE(must_be_list,           is_list,           "list")
+_MUST_BE_TYPE(must_be_dict,           is_dict,           "dict")
+_MUST_BE_TYPE(must_be_pair,           is_pair,           "pair")
+_MUST_BE_TYPE(must_be_atomic,         is_atomic,         "atomic")
+_MUST_BE_TYPE(must_be_number,         is_number,         "number")
+_MUST_BE_TYPE(must_be_acyclic,        is_acyclic,        "acyclic")
+
+#undef _MUST_BE_TYPE
 
 _SWI_CPP2_CPP_inline
 PlModule
@@ -139,7 +176,7 @@ PlGeneralError(PlTerm inside)
 
 _SWI_CPP2_CPP_inline
 PlException
-PlTypeError(const std::string& expected, const PlTerm& actual)
+PlTypeError(const std::string& expected, PlTerm actual)
 { // See PL_type_error()
   return PlGeneralError(PlCompound("type_error",
                                    PlTermv(PlTerm_atom(expected), actual)));
@@ -147,7 +184,7 @@ PlTypeError(const std::string& expected, const PlTerm& actual)
 
 _SWI_CPP2_CPP_inline
 PlException
-PlDomainError(const std::string& expected, const PlTerm& actual)
+PlDomainError(const std::string& expected, PlTerm actual)
 { // See PL_domain_error()
   return PlGeneralError(PlCompound("domain_error",
                                    PlTermv(PlTerm_atom(expected), actual)));
@@ -155,7 +192,7 @@ PlDomainError(const std::string& expected, const PlTerm& actual)
 
 _SWI_CPP2_CPP_inline
 PlException
-PlDomainError(const PlTerm& expected, const PlTerm& actual)
+PlDomainError(PlTerm expected, PlTerm actual)
 { // See PL_domain_error()
   // This is used by
   //    PlDomainError(PlCompound("argv", PlTermv(PlTerm_integer(size_))), ...)
@@ -166,14 +203,14 @@ PlDomainError(const PlTerm& expected, const PlTerm& actual)
 
 _SWI_CPP2_CPP_inline
 PlException
-PlInstantiationError(const PlTerm& t)
+PlInstantiationError(PlTerm t)
 { // See PL_instantiation_error()
   return PlGeneralError(PlCompound("instantiation_error", PlTermv(t)));
 }
 
 _SWI_CPP2_CPP_inline
 PlException
-PlUninstantiationError(const PlTerm& t)
+PlUninstantiationError(PlTerm t)
 { // See PL_uninstantiation_error()
   return PlGeneralError(PlCompound("uninstantiation_error", PlTermv(t)));
 }
@@ -196,7 +233,7 @@ PlExistenceError(const std::string& type, PlTerm actual)
 
 _SWI_CPP2_CPP_inline
 PlException
-PlPermissionError(const std::string& op, const std::string& type, const PlTerm& obj)
+PlPermissionError(const std::string& op, const std::string& type, PlTerm obj)
 { // See: Use PL_permission_error()
   return PlGeneralError(PlCompound("permission_error",
                                    PlTermv(PlTerm_atom(op), PlTerm_atom(type), obj)));
@@ -225,9 +262,19 @@ PlUnknownError(const std::string& description)
 		 *	ATOM IMPLEMENTATION	*
 		 *******************************/
 
+_SWI_CPP2_CPP_inline
+const std::string
+PlAtom::mbchars(unsigned int flags) const
+{ PlStringBuffers _string_buffers;
+  size_t len;
+  char *s;
+  Plx_atom_mbchars(unwrap(), &len, &s, CVT_EXCEPTION|flags);
+  return std::string(s, len);
+}
+
 _SWI_CPP2_CPP_inline
 const std::wstring
-PlAtom::as_wstring() const
+PlAtom::wchars() const
 { PlStringBuffers _string_buffers;
   size_t len;
   const wchar_t *s = Plx_atom_wchars(unwrap(), &len);
@@ -261,7 +308,7 @@ void PlBlob::save(IOSTREAM *fd) const
 _SWI_CPP2_CPP_inline
 PlAtom PlBlob::load(IOSTREAM *fd)
 { (void)PL_warning("Cannot load reference to <%s>", blob_t_->name);
-  PL_fatal_error("Cannot load reference to <%s>", blob_t_->name);
+  PL_system_error("Cannot load reference to <%s>", blob_t_->name);
   return PlAtom(PlAtom::null);
 }
 
@@ -279,10 +326,14 @@ bool PlTerm::unify_blob(const PlBlob* blob) const
 }
 
 _SWI_CPP2_CPP_inline
-bool PlTerm::unify_blob(std::unique_ptr<PlBlob>* blob) const
-{ if ( !PlTerm::unify_blob(blob->get()) )
+bool PlTerm::unify_blob(std::unique_ptr<PlBlob>* b) const
+{ std::unique_ptr<PlBlob> blob(std::move(*b));
+  // if std::move is not supported, the above can be replaced by:
+  //   std:unique_ptr<PlBlob> blob;
+  //   blob.swap(*b);
+  if ( !unify_blob(blob.get()) )
     return false;
-  (void)blob->release(); // Pass ownership to the Prolog blob (`this`)
+  (void)blob.release(); // Pass ownership to the Prolog blob (`this`)
   return true;
 }
 
@@ -300,22 +351,6 @@ PlTerm::copy_term_ref() const
 { return PlTerm(Plx_copy_term_ref(unwrap()));
 }
 
-_SWI_CPP2_CPP_inline
-const std::string
-PlTerm::as_string(PlEncoding enc) const
-{ return get_nchars(CVT_ALL|CVT_WRITEQ|static_cast<unsigned int>(enc));
-}
-
-_SWI_CPP2_CPP_inline
-const std::wstring
-PlTerm::as_wstring() const
-{ wchar_t *s;
-  size_t len;
-  PlStringBuffers _string_buffers;
-  // TODO: split out get_wchars(), similar to get_nchars()
-  PlEx<int>(get_wchars(&len, &s, CVT_ALL|CVT_WRITEQ|BUF_STACK|CVT_EXCEPTION));
-  return std::wstring(s, len);
-}
 
 _SWI_CPP2_CPP_inline
 void
@@ -371,7 +406,7 @@ PlTerm::PlTerm(const PlRecord& r)
 		 *******************************/
 
 _SWI_CPP2_CPP_inline
-PlTerm_tail::PlTerm_tail(const PlTerm& l)
+PlTerm_tail::PlTerm_tail(PlTerm l)
 { if ( l.is_variable() || l.is_list() )
     reset(l.copy_term_ref());
   else
@@ -408,7 +443,7 @@ PlRewindOnFail(std::function<bool()> f)
 { PlFrame frame;
   bool rc = f();
   if ( !rc )
-    frame.rewind();
+    frame.discard();  // Same as frame.rewind(); destructor's frame.close()
   return rc;
 }
 
@@ -588,15 +623,25 @@ bool
 PlTerm::eq(const std::string& s) const
 { char *s0;
 
+  // Doesn't handle non-NUL terminated - but it's only used by deprecated operator ==
   if ( get_chars(&s0, CVT_ALL) )
-    return s.compare(s0) == 0; // Doesn't handle non-NUL terminated - but it's a deprecated method
+    return s.compare(s0) == 0; 
 
   throw PlTypeError("text", *this);
 }
 
 _SWI_CPP2_CPP_inline
 bool
-PlTerm::eq(const PlAtom& a) const
+PlTerm::eq(const std::wstring& s) const
+{ // Doesn't handle non-NUL terminated - but it's only used by deprecated operator ==
+  return s.compare(get_wchars(CVT_ALL)) == 0;
+
+  throw PlTypeError("text", *this);
+}
+
+_SWI_CPP2_CPP_inline
+bool
+PlTerm::eq(PlAtom a) const
 { atom_t v;
 
   if ( Plx_get_atom(unwrap(), &v) )
@@ -670,21 +715,21 @@ PlCompound::PlCompound(const std::wstring& functor, const PlTermv& args)
 		 *******************************/
 
 _SWI_CPP2_CPP_inline
-PlTermv::PlTermv(const PlAtom& a)
+PlTermv::PlTermv(PlAtom a)
   : size_(1),
     a0_(PlTerm_atom(a).unwrap())
 { PlEx<bool>(a0_ != (term_t)0);
 }
 
 _SWI_CPP2_CPP_inline
-PlTermv::PlTermv(const PlTerm& m0)
+PlTermv::PlTermv(PlTerm m0)
   : size_(1),
     a0_(m0.unwrap())
 { // Assume that m0 is valid
 }
 
 _SWI_CPP2_CPP_inline
-PlTermv::PlTermv(const PlTerm& m0, const PlTerm& m1)
+PlTermv::PlTermv(PlTerm m0, PlTerm m1)
   : size_(2),
     a0_(Plx_new_term_refs(2))
 { PlEx<bool>(a0_ != (term_t)0);
@@ -693,7 +738,7 @@ PlTermv::PlTermv(const PlTerm& m0, const PlTerm& m1)
 }
 
 _SWI_CPP2_CPP_inline
-PlTermv::PlTermv(const PlTerm& m0, const PlTerm& m1, const PlTerm& m2)
+PlTermv::PlTermv(PlTerm m0, PlTerm m1, PlTerm m2)
   : size_(3),
     a0_(Plx_new_term_refs(3))
 { PlEx<bool>(a0_ != (term_t)0);
@@ -703,7 +748,7 @@ PlTermv::PlTermv(const PlTerm& m0, const PlTerm& m1, const PlTerm& m2)
 }
 
 _SWI_CPP2_CPP_inline
-PlTermv::PlTermv(const PlTerm& m0, const PlTerm& m1, const PlTerm& m2, const PlTerm& m3)
+PlTermv::PlTermv(PlTerm m0, PlTerm m1, PlTerm m2, PlTerm m3)
   : size_(4),
     a0_(Plx_new_term_refs(4))
 { PlEx<bool>(a0_ != (term_t)0);
@@ -714,8 +759,7 @@ PlTermv::PlTermv(const PlTerm& m0, const PlTerm& m1, const PlTerm& m2, const PlT
 }
 
 _SWI_CPP2_CPP_inline
-PlTermv::PlTermv(const PlTerm& m0, const PlTerm& m1, const PlTerm& m2,
-                 const PlTerm& m3, const PlTerm& m4)
+PlTermv::PlTermv(PlTerm m0, PlTerm m1, PlTerm m2, PlTerm m3, PlTerm m4)
   : size_(5),
     a0_(Plx_new_term_refs(5))
 { PlEx<bool>(a0_ != (term_t)0);
@@ -920,7 +964,7 @@ PlStream::defn \
 }
 
 _SWI_CPP2_CPP_check_rc(int, set_timeout(int tmo), Sset_timeout(s_, tmo))
-_SWI_CPP2_CPP_check_rc(int, unit_size(), Sunit_size(s_));
+_SWI_CPP2_CPP_check_rc(int, unit_size(), Sunit_size(s_))
 _SWI_CPP2_CPP_nocheck(bool, canrepresent(int c), Scanrepresent(c, s_))
 _SWI_CPP2_CPP_check_rc(int, putcode(int c), Sputcode(c, s_))
 _SWI_CPP2_CPP_check_rc(int, getcode(), Sgetcode(s_))
@@ -938,8 +982,8 @@ _SWI_CPP2_CPP_check_rc(int, setenc(IOENC new_enc, IOENC *old_enc), Ssetenc(s_, n
 _SWI_CPP2_CPP_check_rc(int, setlocale(struct PL_locale *new_loc, struct PL_locale **old_loc), Ssetlocale(s_, new_loc, old_loc))
 _SWI_CPP2_CPP_check_rc(int, flush(), Sflush(s_))
 _SWI_CPP2_CPP_check_rc(int64_t, size(), Ssize(s_))
-_SWI_CPP2_CPP_check_rc(int, seek(long pos, int whence), Sseek(s_, pos, whence))
-_SWI_CPP2_CPP_check_rc(int, tell(), Stell(s_))
+_SWI_CPP2_CPP_check_rc(int, seek(int64_t pos, int whence), Sseek64(s_, pos, whence))
+_SWI_CPP2_CPP_check_rc(int64_t, tell(), Stell64(s_))
 _SWI_CPP2_CPP_check_rc(int, close(), Sclose(s_))
 _SWI_CPP2_CPP_check_rc(int, gcclose(int flags), Sgcclose(s_, flags))
 _SWI_CPP2_CPP_check_rc(ssize_t, read_pending(char *buf, size_t limit, int flags), Sread_pending(s_, buf, limit, flags))
