@@ -64,12 +64,14 @@
 	    [ print_html/2, print_html/1, page/4, html/3,
 	      html_print_length/2
 	    ]).
+:- if(exists_source(http_exception)).
 :- autoload(http_exception,[map_exception_to_http_status/4]).
+:- endif.
 :- autoload(mimepack,[mime_pack/3]).
 :- autoload(mimetype,[file_mime_type/2]).
 :- autoload(library(apply),[maplist/2]).
 :- autoload(library(base64),[base64/2]).
-:- autoload(library(debug),[debug/3,debugging/1]).
+:- use_module(library(debug),[debug/3,debugging/1]).
 :- autoload(library(error),[syntax_error/1,domain_error/2]).
 :- autoload(library(lists),[append/3,member/2,select/3,delete/3]).
 :- autoload(library(memfile),
@@ -248,12 +250,18 @@ http_reply(Data, Out, HdrExtra, _Context, Request, Code) :-
         throw(error(http_write_short(Data, Sent), _))
     ;   E = error(timeout_error(write, _), _)
     ->  throw(E)
-    ;   map_exception_to_http_status(E, Status, NewHdr, NewContext),
-        http_status_reply(Status, Out, NewHdr, NewContext, Request, Code)
+    ;   map_exception_to_http_status(E, Status, NewHdr, NewContext)
+    ->  http_status_reply(Status, Out, NewHdr, NewContext, Request, Code)
+    ;   throw(E)
     ).
 http_reply(Status, Out, HdrExtra, Context, Request, Code) :-
     http_status_reply(Status, Out, HdrExtra, Context, Request, Code).
 
+:- if(\+current_predicate(map_exception_to_http_status/4)).
+map_exception_to_http_status(_E, _Status, _NewHdr, _NewContext) :-
+    fail.
+:- endif.
+
 :- meta_predicate
     if_no_head(0, +).
 
@@ -993,6 +1001,11 @@ content_length_in_encoding(Enc, Stream, Bytes) :-
 %     * atom(+Type, +Atom)
 %     Send Atom using the indicated MIME-type.
 %
+%     * string(+String)
+%     * string(+Type, +String)
+%     Similar to atom(Atom) and atom(Type,Atom), accepting a SWI-Prolog
+%     string.
+%
 %     * cgi_stream(+Stream, +Len) Read the input from Stream which,
 %     like CGI data starts with a partial HTTP header. The fields of
 %     this header are merged with the provided HdrExtra fields. The
