@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2023, University of Amsterdam
+    Copyright (c)  2011-2024, University of Amsterdam
 			      VU University Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -126,9 +126,7 @@ outchr(format_state *state, int chr)
       for(s=buf; s<e; s++)
 	addBuffer((Buffer)&state->buffer, *s, char);
     } else
-    { char c = chr;
-
-      addBuffer((Buffer)&state->buffer, c, char);
+    { addBuffer((Buffer)&state->buffer, (char)chr, char);
     }
 
     state->buffered++;
@@ -251,9 +249,9 @@ PRED_IMPL("format_predicate", 2, format_predicate, META)
 		    A2);
 
   if ( !format_predicates )
-    format_predicates = newHTable(8);
+    format_predicates = newHTableWP(8);
 
-  updateHTable(format_predicates, (void *)(intptr_t)c, proc);
+  updateHTableWP(format_predicates, c, proc);
 
   return TRUE;
 }
@@ -262,8 +260,6 @@ PRED_IMPL("format_predicate", 2, format_predicate, META)
 static
 PRED_IMPL("current_format_predicate", 2, current_format_predicate, NDET|META)
 { PRED_LD
-  intptr_t name;
-  predicate_t pred;
   TableEnum e;
   fid_t fid;
 
@@ -274,7 +270,7 @@ PRED_IMPL("current_format_predicate", 2, current_format_predicate, NDET|META)
   { case FRG_FIRST_CALL:
       if ( !format_predicates )
 	fail;
-      e = newTableEnum(format_predicates);
+      e = newTableEnumWP(format_predicates);
       break;
     case FRG_REDO:
       e = CTX_PTR;
@@ -290,8 +286,14 @@ PRED_IMPL("current_format_predicate", 2, current_format_predicate, NDET|META)
   { freeTableEnum(e);
     return FALSE;
   }
-  while( advanceTableEnum(e, (void**)&name, (void**)&pred) )
-  { if ( PL_unify_integer(chr, name) &&
+
+  table_key_t tk;
+  table_value_t tv;
+  while( advanceTableEnum(e, &tk, &tv) )
+  { int c = (int)tv;
+    predicate_t pred = val2ptr(tv);
+
+    if ( PL_unify_integer(chr, c) &&
 	 PL_unify_predicate(descr, pred, 0) )
     { PL_close_foreign_frame(fid);
       ForeignRedoPtr(e);
@@ -306,7 +308,7 @@ PRED_IMPL("current_format_predicate", 2, current_format_predicate, NDET|META)
 }
 
 
-static word
+static foreign_t
 format_impl(IOSTREAM *out, term_t format, term_t Args, Module m)
 { GET_LD
   term_t argv;
@@ -351,10 +353,10 @@ format_impl(IOSTREAM *out, term_t format, term_t Args, Module m)
 
 #define format(out, fmt, args) LDFUNC(format, out, fmt, args)
 
-static word
+static foreign_t
 format(DECL_LD term_t out, term_t format, term_t args)
 { redir_context ctx;
-  word rc;
+  foreign_t rc;
   Module m = NULL;
   term_t list = PL_new_term_ref();
 
@@ -528,7 +530,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 
 					/* Check for user defined format */
 	  if ( format_predicates &&
-	       (proc = lookupHTable(format_predicates, (void*)((intptr_t)c))) )
+	       (proc = lookupHTableWP(format_predicates, c)) )
 	  { size_t arity;
 	    term_t av;
 	    sub_state sstate;
@@ -608,7 +610,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		  if ( !valueExpression(argv, &n) )
 		  { char f[2];
 
-		    f[0] = c;
+		    f[0] = (char)c;
 		    f[1] = EOS;
 		    AR_CLEANUP();
 		    FMT_ARG(f, argv); /* returns error */
@@ -640,6 +642,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		{ AR_CTX
 		  number i;
 		  tmp_buffer b;
+		  char *si;
 
 		  NEED_ARG;
 		  AR_BEGIN();
@@ -647,7 +650,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		       !toIntegerNumber(&i, 0) )
 		  { char f[2];
 
-		    f[0] = c;
+		    f[0] = (char)c;
 		    f[1] = EOS;
 		    AR_CLEANUP();
 		    FMT_ARG(f, argv);
@@ -672,19 +675,17 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 
 		    if ( arg == DEFAULT )
 		      arg = 0;
-		    if ( !formatInteger(l, arg, 10, TRUE, &i, (Buffer)&b) )
-		      FMT_EXEPTION();
+		    si = formatInteger(l, arg, 10, TRUE, &i, (Buffer)&b);
 		  } else if ( c == 'I' )
 		  { PL_locale ltmp;
 		    char grouping[2];
 
-		    grouping[0] = (arg == DEFAULT ? 3 : arg);
+		    grouping[0] = (char)(arg == DEFAULT ? 3 : arg);
 		    grouping[1] = '\0';
 		    ltmp.thousands_sep = L"_";
 		    ltmp.grouping = grouping;
 
-		    if ( !formatInteger(&ltmp, 0, 10, TRUE, &i, (Buffer)&b) )
-		      FMT_EXEPTION();
+		    si = formatInteger(&ltmp, 0, 10, TRUE, &i, (Buffer)&b);
 		  } else			/* r,R */
 		  { if ( arg == DEFAULT )
 		      arg = 8;
@@ -692,15 +693,17 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		    { term_t r = PL_new_term_ref();
 
 		      PL_put_integer(r, arg);
-		      return PL_error(NULL, 0, NULL, ERR_DOMAIN,
-				      ATOM_radix, r);
-		    }
-		    if ( !formatInteger(NULL, 0, arg, c == 'r', &i, (Buffer)&b) )
-		      FMT_EXEPTION();
+		      PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_radix, r);
+		      si = NULL;
+		    } else
+		      si = formatInteger(NULL, 0, arg, c == 'r', &i, (Buffer)&b);
 		  }
 		  clearNumber(&i);
 		  AR_END();
-		  rc = oututf80(&state, baseBuffer(&b, char));
+		  if ( si )
+		    rc = oututf80(&state, si);
+		  else
+		    rc = FALSE;
 		  discardBuffer(&b);
 		  if ( !rc )
 		    goto out;
@@ -1008,7 +1011,7 @@ lappend(const wchar_t *l, int def, Buffer out)
     { int c = *e;
 
       if ( c < 128 )
-      { addBuffer(out, c, char);
+      { addBuffer(out, (char)c, char);
       } else
       { char buf[6];
 	char *e8, *s;
@@ -1020,7 +1023,7 @@ lappend(const wchar_t *l, int def, Buffer out)
       }
     }
   } else
-  { addBuffer(out, def, char);
+  { addBuffer(out, (char)def, char);
   }
 }
 
@@ -1029,7 +1032,7 @@ revert_string(char *s, size_t len)
 { char *e = &s[len-1];
 
   for(; e>s; s++,e--)
-  { int c = *e;
+  { char c = *e;
 
     *e = *s;
     *s = c;
@@ -1102,22 +1105,21 @@ formatInteger(PL_locale *locale, int div, int radix, bool smll, Number i,
 #ifdef O_BIGNUM
     case V_MPZ:
     { GET_LD
-      size_t len = (double)mpz_sizeinbase(i->value.mpz, 2) * log(radix)/log(2) * 1.2;
+	size_t len = (size_t)((double)mpz_sizeinbase(i->value.mpz, 2) *
+			      log(radix)/log(2) * 1.2);
       char tmp[256];
       char *buf;
-      int rc = TRUE;
+      int rc;
 
       if ( len+2 > sizeof(tmp) )
-	buf = PL_malloc(len+2);
+	buf = tmp_malloc(len+2);
       else
 	buf = tmp;
 
-      EXCEPTION_GUARDED({ LD->gmp.persistent++;
-			  mpz_get_str(buf, radix, i->value.mpz);
-			  LD->gmp.persistent--;
+      EXCEPTION_GUARDED({ mpz_get_str(buf, radix, i->value.mpz);
+			  rc = TRUE;
 			},
-			{ LD->gmp.persistent--;
-			  rc = PL_rethrow();
+			{ rc = FALSE;
 			});
       if ( !rc )
 	return NULL;
@@ -1126,7 +1128,7 @@ formatInteger(PL_locale *locale, int div, int radix, bool smll, Number i,
       { char *s;
 
 	for(s=buf; *s; s++)
-	  *s = toupper(*s);
+	  *s = (char)toupper(*s);
       }
 
       if ( grouping || div > 0 )
@@ -1162,7 +1164,7 @@ formatInteger(PL_locale *locale, int div, int radix, bool smll, Number i,
       }
 
       if ( buf != tmp )
-	PL_free(buf);
+	tmp_free(buf);
 
       addBuffer(out, EOS, char);
       return baseBuffer(out, char);
@@ -1424,7 +1426,7 @@ formatFloat(PL_locale *locale, int how, int arg, Number f, Buffer out)
     }
     case V_MPQ:
     { char tmp[12];
-      int size;
+      size_t size;
       int written;
       int fbits;
       int digits;
@@ -1450,7 +1452,7 @@ formatFloat(PL_locale *locale, int how, int arg, Number f, Buffer out)
 	print_mpz_f:
 
 	  if ( mpz_sgn(t1) != 0 )
-	  { size = mpz_sizeinbase(t1, 2) * log(10)/log(2) * 1.2 + 1;
+	  { size =(size_t)((double)mpz_sizeinbase(t1, 2) * log(10)/log(2) * 1.2 + 1);
 	    if ( !growBuffer(out, size) )
 	    { PL_no_memory();
 	      return NULL;
@@ -1489,7 +1491,7 @@ formatFloat(PL_locale *locale, int how, int arg, Number f, Buffer out)
 	  if (arg)
 	  { memmove(out->base+written-(arg-1), out->base+written-arg, arg+1);
 	    if ( locale->decimal_point && locale->decimal_point[0] )
-	      *(out->base+written-arg) = locale->decimal_point[0];
+	      *(out->base+written-arg) = (char)locale->decimal_point[0];
 	    else
 	      *(out->base+written-arg) = '.';
 	    written++;
@@ -1579,7 +1581,7 @@ formatFloat(PL_locale *locale, int how, int arg, Number f, Buffer out)
 	  if (arg)
 	  { memmove(out->base+2, out->base+1, written+1);
 	    if ( locale->decimal_point && locale->decimal_point[0] )
-	      *(out->base+1) = locale->decimal_point[0];
+	      *(out->base+1) = (char)locale->decimal_point[0];
 	    else
 	      *(out->base+1) = '.';
 	    written++;
